/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.defensesolutions;


import com.esri.arcgis.defensesolutions.*;
import com.esri.arcgis.geometry.*;
import com.esri.arcgis.system.*;
import com.esri.arcgis.display.*;
import java.io.IOException;
import com.esri.arcgis.interop.AutomationException;

/**
 * Demonstrates a simple leadering style and the required elements including:
 *
 * 1. A Line style to be used for drawing leadered graphics
 * 2. A Construction routine for
 * 2.a. Moving leadered graphics to their offset position
 * 2.b. Creating the leadering line between leadered graphics and the callout point
 */
public class HBarLdrStyle implements IGraphicLeaderStyle, IClone, IHBarLdrStyle, IPersistVariant {
	private static final long serialVersionUID = 1L;
	ILineSymbol mLineSymbol = null;
    public HBarLdrStyle() {

    	try {

	    	// Create the Line Symbol to be used for leader lines
	    	mLineSymbol = new SimpleLineSymbol();
	    	mLineSymbol.setWidth(2.0);

	    	IRgbColor color = new RgbColor();
	    	color.setRGB(0); // black
	    	mLineSymbol.setColor((IColor)color);
    	}
    	catch (Exception ex) {
    		ex.printStackTrace();
    	}
    }

    public ILineSymbol getLineSymbol() throws IOException, AutomationException {
        return mLineSymbol;
    }

    public void setLineSymbolByRef(ILineSymbol iLineSymbol) throws IOException,
            AutomationException {
        mLineSymbol = iLineSymbol;
    }

    public IFillSymbol getFillSymbol() throws IOException, AutomationException {

        return null;
    }

    public void setFillSymbolByRef(IFillSymbol iFillSymbol) throws IOException,
            AutomationException {
    }

    public IMarkerSymbol getMarkerSymbol() throws IOException,
            AutomationException {
        return null;
    }

    public void setMarkerSymbolByRef(IMarkerSymbol iMarkerSymbol) throws
            IOException, AutomationException {
    }

    public void drawThumbnail(Object object) throws IOException,
            AutomationException {

    }

    /**
     * During the construct phase, it is the responsibility of the
     * Leader Style to do 2 things:
     *
     * 1. Set the AnchorPoints of the leadered children cached graphics
     * 2. Set the IGraphicLeader.Geometry to the leadering line
     */
    public void construct(IDisplay iDisplay, IGraphicLeader iGraphicLeader) throws
            IOException, AutomationException {
    try{

    	IGeometryCollection polyline = new Polyline();
        // ICachedGraphic cachedGraphic;
        // ILeaderable leaderable;

        double ANCHOR_POINT_OFFSET = 1.0;

        int leaderedChildren = iGraphicLeader.getCount();

        double sumX = 0.0, sumY = 0.0;

        // Move the leadered children to some arbitrary offset
        for (int i = 0; i<leaderedChildren; i++)
        {
        	ICachedGraphic cachedGraphic = (ICachedGraphic)iGraphicLeader.item(i);

        	IGeometry geo = cachedGraphic.ICachedGraphic_getGeometry();
        	IPoint preLeaderedPoint = (IPoint)geo;

        	double pointX, pointY;
        	pointX = preLeaderedPoint.getX();
        	pointY = preLeaderedPoint.getY();

        	// Move te graphic over by some constant
        	IPoint postLeaderedPoint = new Point();
        	postLeaderedPoint.putCoords(pointX + ANCHOR_POINT_OFFSET, pointY);

        	// Set the Anchor Point (this will move the CachedGraphics)
        	IGraphicCallout callout = (IGraphicCallout)cachedGraphic;
        	callout.setAnchorPointByRef(postLeaderedPoint);
        	cachedGraphic.refresh(iDisplay);

        	sumX += pointX;
        	sumY += pointY;
        }

        // compute the average point / center of mass for the graphics
        double averageX = sumX/leaderedChildren;
        double averageY = sumY/leaderedChildren;

        IPoint averagePoint = new Point();
        averagePoint.putCoords(averageX, averageY);

        // Now create the leadering line to simply a line from the graphics to the
        // average point
        for (int i = 0; i<leaderedChildren; i++)
        {
        	// Get the Anchor Point for each child graphic
        	ICachedGraphic cachedGraphic = (ICachedGraphic)iGraphicLeader.item(i);
        	IGraphicCallout callout = (IGraphicCallout)cachedGraphic;
        	IPoint postLeaderedPoint = callout.getAnchorPoint();

        	// Now draw a line from the center point to the graphci
            IPointCollection pPath = new Path();

            pPath.addPoint(averagePoint, null, null);
            pPath.addPoint(postLeaderedPoint, null, null);

            polyline.addGeometry((IGeometry)pPath, null, null);
        }

        // Now set the leaders geometry
    	IGeometryBag pGeoBag;
    	pGeoBag = iGraphicLeader.getGeometry();
    	pGeoBag.setEmpty();

    	IGeometryCollection pGeomCol = (IGeometryCollection)pGeoBag;
    	pGeomCol.addGeometry((IGeometry)polyline, null, null);

        }
        catch(Exception e){e.printStackTrace();}

    }

    public String getName() throws IOException, AutomationException {
        return "";
    }

    public String getDescription() throws IOException, AutomationException {
        return "";
    }

    public String getCLSID() throws IOException, AutomationException {
        return "";
    }

    public String getPicture() throws IOException, AutomationException {
        return "";
    }

    public IClone esri_clone() throws IOException, AutomationException {
        return null;
    }

    public void assign(IClone iClone) throws IOException, AutomationException {
    }

    public boolean isEqual(IClone iClone) throws IOException,
            AutomationException {
        return false;
    }

    public boolean isIdentical(IClone iClone) throws IOException,
            AutomationException {
        return false;
    }

    public IUID getID() throws IOException, AutomationException {
        return null;
    }

    public void load(IVariantStream iVariantStream) throws IOException,
            AutomationException {
    }

    public void save(IVariantStream iVariantStream) throws IOException,
            AutomationException {
    }
}
